package q1k2.base.common.ceco.helper;

import kd.bos.util.StringUtils;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.Iterator;
import java.util.List;

/**
 * @BelongsProject: q1k2-cosmic
 * @BelongsPackage: q1k2.base.common.ceco.helper
 * @Author: yanhongwei
 * @CreateTime: 2024-06-26  18:00
 * @Description: TODO
 * @Version: 1.0
 */
public class XMLFileHelper {
    private String xmlFileName;
    private Document xmlDocument;

    public XMLFileHelper(String xmlFileName) throws IOException, DocumentException {
        this.xmlFileName = xmlFileName;
        if (!StringUtils.isEmpty(this.xmlFileName)) {
            File file = new File(this.xmlFileName);
            if (file.exists() && !file.isDirectory()) {
                SAXReader reader = new SAXReader();
                this.xmlDocument = reader.read(file);
            } else {
                throw new IOException("文件" + this.xmlFileName + "不存在，或格式错误");
            }
        }
    }

    public boolean doc2XmlFile(Document document, String filename) throws IOException {
        if (document != null && !StringUtils.isEmpty(filename)) {
            XMLWriter writer = new XMLWriter(new OutputStreamWriter(new FileOutputStream(filename), "UTF-8"));
            writer.write(document);
            writer.close();
            return true;
        } else {
            return false;
        }
    }

    public Element getRootElement() {
        return this.xmlDocument.getRootElement();
    }

    public String getAttributeValue(String elementKeys) {
        if (StringUtils.isEmpty(elementKeys)) {
            return "";
        } else {
            String[] elementKeyList = elementKeys.split("\\.");
            Element element = this.getRootElement();

            for(int i = 0; i < elementKeyList.length; ++i) {
                String elementKey = elementKeyList[i];
                if (i == elementKeyList.length - 1) {
                    return element.attributeValue(elementKey);
                }

                element = element.element(elementKey);
                if (element == null) {
                    return "";
                }
            }

            return "";
        }
    }

    public String getAttributeValue(Element element, String name) {
        return element != null && !StringUtils.isEmpty(name) ? element.attributeValue(name) : "";
    }

    public String getElementStringValue(Element element, String name) {
        if (element != null && !StringUtils.isEmpty(name)) {
            Element ele = element.element(name);
            return ele != null ? ele.getStringValue() : "";
        } else {
            return "";
        }
    }

    public boolean getElementBooleanValue(Element element, String name) {
        if (element != null && !StringUtils.isEmpty(name)) {
            String value = element.element(name).getStringValue();
            return !StringUtils.isEmpty(value) && ("1".equalsIgnoreCase(value) || "true".equalsIgnoreCase(value));
        } else {
            return false;
        }
    }

    public Element getElement(String elementKeys) {
        Element rootElement = this.getRootElement();
        if (StringUtils.isEmpty(elementKeys)) {
            return rootElement;
        } else {
            String[] elementKeyList = elementKeys.split("\\.");
            Element element = null;

            for(int i = 0; i < elementKeyList.length; ++i) {
                String elementKey = elementKeyList[i];
                if (element == null) {
                    element = rootElement.element(elementKey);
                    if (element == null) {
                        return null;
                    }
                } else {
                    element = element.element(elementKey);
                    if (element == null) {
                        return null;
                    }
                }
            }

            return element;
        }
    }

    public long getElementLongValue(String elementKeys) {
        String value = this.getElementStringValue(elementKeys);
        return StringUtils.isEmpty(value) ? 0L : Long.parseLong(value);
    }

    public String getElementStringValue(String elementKeys) {
        if (StringUtils.isEmpty(elementKeys)) {
            return "";
        } else {
            Element rootElement = this.getRootElement();
            String[] elementKeyList = elementKeys.split("\\.");
            Element element = null;

            for(int i = 0; i < elementKeyList.length; ++i) {
                String elementKey = elementKeyList[i];
                if (element == null) {
                    element = rootElement.element(elementKey);
                    if (element == null) {
                        return "";
                    }
                } else {
                    element = element.element(elementKey);
                    if (element == null) {
                        return "";
                    }
                }
            }

            if (element == null) {
                return "";
            } else {
                return element.getStringValue();
            }
        }
    }

    public List<Element> getElements(String elementKeys) {
        if (StringUtils.isEmpty(elementKeys)) {
            return null;
        } else {
            Element rootElement = this.getRootElement();
            String[] elementKeyList = elementKeys.split("\\.");
            Element element = null;
            if (elementKeyList.length == 1) {
                return rootElement.elements(elementKeys);
            } else {
                for(int i = 0; i < elementKeyList.length; ++i) {
                    String elementKey = elementKeyList[i];
                    if (i >= elementKeyList.length - 1) {
                        if (element == null) {
                            return null;
                        }

                        return element.elements(elementKey);
                    }

                    if (element == null) {
                        element = rootElement.element(elementKey);
                        if (element == null) {
                            return null;
                        }
                    } else {
                        element = element.element(elementKey);
                        if (element == null) {
                            return null;
                        }
                    }
                }

                return null;
            }
        }
    }

    public List<Element> getElements(Element element, String elementKey) {
        return element != null && !StringUtils.isEmpty(elementKey) ? element.elements(elementKey) : null;
    }

    public Element getElementFromID(List<Element> kdpkgElements, String idValue) {
        if (kdpkgElements != null && !StringUtils.isEmpty(idValue)) {
            Iterator var3 = kdpkgElements.iterator();

            Element element;
            do {
                if (!var3.hasNext()) {
                    return null;
                }

                element = (Element)var3.next();
            } while(!idValue.equalsIgnoreCase(element.elementText("ID")));

            return element;
        } else {
            return null;
        }
    }
}
