package q1k2.base.common.ceco.model.xmlfilemodel;

import com.alibaba.fastjson.JSONObject;
import kd.bos.servicehelper.DBServiceHelper;
import kd.bos.util.StringUtils;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import q1k2.base.common.ceco.helper.XMLFileHelper;
import q1k2.base.common.ceco.model.AppInfo;
import q1k2.base.common.ceco.model.AppMenuInfo;
import q1k2.base.common.utils.FileUtils;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * @BelongsProject: q1k2-cosmic
 * @BelongsPackage: q1k2.base.common.ceco.model.xmlfilemodel
 * @Author: yanhongwei
 * @CreateTime: 2024-06-26  17:58
 * @Description: TODO
 * @Version: 1.0
 */
public class AppXmlFileInfo {
    private static final String TABLE_APPMENU = "t_isv_patchpack_appmenu";
    private static final String KEYPREFIX = "pkKey_";
    private String dmZipFile;
    private String filePath;
    private String fileName;
    private XMLFileHelper metadata;
    private XMLFileHelper metadata_cn;
    private JSONObject menuParentPKIdJSON;

    private void addMenuParentPKID(String key, long value) {
        if (this.menuParentPKIdJSON == null) {
            this.menuParentPKIdJSON = new JSONObject();
        }

        if (!StringUtils.isEmpty(key)) {
            key = "pkKey_" + key;
            if (!this.menuParentPKIdJSON.containsKey(key)) {
                this.menuParentPKIdJSON.put(key, value);
            }

        }
    }

    public long getMenuParentPKId(String key) {
        if (!StringUtils.isEmpty(key) && this.menuParentPKIdJSON != null && this.menuParentPKIdJSON.containsKey("pkKey_" + key)) {
            key = "pkKey_" + key;
            return this.menuParentPKIdJSON.getLongValue(key);
        } else {
            return 0L;
        }
    }

    public AppXmlFileInfo(String dmZipFile, String filePath, String fileName) throws DocumentException, IOException {
        this.dmZipFile = dmZipFile;
        this.filePath = filePath;
        this.fileName = fileName;
        if (!StringUtils.isEmpty(filePath) && !StringUtils.isEmpty(fileName)) {
            String fileName_ch = this.fileName.split("\\.app")[0] + ".zh_CN.appx";
            this.metadata = new XMLFileHelper(filePath + fileName);
            if (FileUtils.checkFileExists(this.filePath + fileName_ch)) {
                this.metadata_cn = new XMLFileHelper(this.filePath + fileName_ch);
            }
        }

    }

    public AppInfo getAppInfo() {
        if (this.metadata == null) {
            return null;
        } else {
            AppInfo appInfo = new AppInfo();
            appInfo.setDmzip(this.dmZipFile);
            appInfo.setAppid(this.metadata.getElementStringValue("Id"));
            appInfo.setMasterId(this.metadata.getElementStringValue("MasterId"));
            appInfo.setAppnumber(this.metadata.getElementStringValue("DesignMetas.DesignAppMeta.Number"));
            String isvCode = this.metadata.getElementStringValue("DesignMetas.DesignAppMeta.Isv");
            appInfo.setAppisv(isvCode);
            appInfo.setDbroute(this.metadata.getElementStringValue("DesignMetas.DesignAppMeta.DbRoute"));
            String allUserApp = this.metadata.getElementStringValue("DesignMetas.DesignAppMeta.AllUserApp");
            appInfo.setIsall(!StringUtils.isEmpty(allUserApp) && "1".equalsIgnoreCase(allUserApp));
            String devType = this.metadata.getElementStringValue("DesignMetas.DesignAppMeta.DevType");
            appInfo.setIsextendapp(!StringUtils.isEmpty(devType) && "2".equalsIgnoreCase(devType));
            if (this.metadata_cn != null) {
                appInfo.setAppname(this.metadata_cn.getElementStringValue("DesignMetas.DesignAppMetaL.Name"));
            }

            appInfo.setAppname_en("");
            return appInfo;
        }
    }

    public List<AppMenuInfo> getAppMenuInfos() {
        List<AppMenuInfo> appMenuInfoList = new ArrayList();
        if (this.metadata == null) {
            return appMenuInfoList;
        } else {
            AppInfo appInfo = this.getAppInfo();
            long appPKId = DBServiceHelper.genLongIds("t_isv_patchpack_appmenu", 1)[0];
            this.addMenuParentPKID(appInfo.getAppid(), appPKId);
            this.addMenuParentPKID(appInfo.getMasterId(), appPKId);
            AppMenuInfo menuApp = new AppMenuInfo();
            menuApp.setPKID(appPKId);
            menuApp.setParentPKID(0L);
            menuApp.setId(appInfo.getAppid());
            menuApp.setNumber(appInfo.getAppnumber());
            String tagStr = "";
            if (appInfo.isIsextendapp()) {
                tagStr = "（扩展）";
            }

            menuApp.setName(appInfo.getAppname() + tagStr);
            menuApp.setCaption(appInfo.getAppname());
            menuApp.setParentId("");
            menuApp.setVisible(true);
            appMenuInfoList.add(menuApp);
            List<Element> appMenus = this.metadata.getElements("DesignMetas.DesignAppMeta.DataXml.AppMetadata.AppMenus.AppMenuElement");
            if (appMenus != null && !appMenus.isEmpty()) {
                Iterator var8 = appMenus.iterator();

                while(var8.hasNext()) {
                    Element menuElement = (Element)var8.next();
                    String menuId = this.metadata.getElementStringValue(menuElement, "Id");
                    if (!StringUtils.isEmpty(menuId)) {
                        long pkId = DBServiceHelper.genLongIds("t_isv_patchpack_appmenu", 1)[0];
                        AppMenuInfo menuInfo = new AppMenuInfo();
                        menuInfo.setPKID(pkId);
                        menuInfo.setFormId(this.metadata.getElementStringValue(menuElement, "FormId"));
                        menuInfo.setVisible(this.metadata.getElementBooleanValue(menuElement, "Visible"));
                        String parentId = this.metadata.getElementStringValue(menuElement, "ParentId");
                        menuInfo.setParentId(parentId);
                        long parentPKID = 0L;
                        this.addMenuParentPKID(menuId, pkId);
                        menuInfo.setParentPKID(parentPKID);
                        menuInfo.setOpenType(this.metadata.getElementStringValue(menuElement, "OpenType"));
                        menuInfo.setNumber(this.metadata.getElementStringValue(menuElement, "Number"));
                        menuInfo.setFormName(this.metadata.getElementStringValue(menuElement, "FormName"));
                        menuInfo.setFormNumber(this.metadata.getElementStringValue(menuElement, "FormNumber"));
                        menuInfo.setName(this.metadata.getElementStringValue(menuElement, "Name"));
                        menuInfo.setCaption(this.metadata.getElementStringValue(menuElement, "Caption"));
                        menuInfo.setId(menuId);
                        appMenuInfoList.add(menuInfo);
                    }
                }

                return appMenuInfoList;
            } else {
                return appMenuInfoList;
            }
        }
    }

    public String getFilePath() {
        return this.filePath;
    }

    public String getFileName() {
        return this.fileName;
    }

    public XMLFileHelper getMetadata() {
        return this.metadata;
    }

    public XMLFileHelper getMetadata_cn() {
        return this.metadata_cn;
    }

    public String getDmZipFile() {
        return this.dmZipFile;
    }
}
