package q1k2.base.common.ceco.model.xmlfilemodel;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.util.StringUtils;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import q1k2.base.common.ceco.helper.XMLFileHelper;
import q1k2.base.common.ceco.model.DymInfo;
import q1k2.base.common.utils.FileUtils;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * @BelongsProject: q1k2-cosmic
 * @BelongsPackage: q1k2.base.common.ceco.model.xmlfilemodel
 * @Author: yanhongwei
 * @CreateTime: 2024-06-26  18:09
 * @Description: TODO
 * @Version: 1.0
 */
public class DymXmlFileInfo {
    private String dmZipFile;
    private String filePath;
    private String fileName;
    private XMLFileHelper metadata;
    private XMLFileHelper metadata_cn;

    public DymXmlFileInfo(String dmZipFile, String filePath, String fileName) throws DocumentException, IOException {
        this.dmZipFile = dmZipFile;
        this.filePath = filePath;
        this.fileName = fileName;
        if (!StringUtils.isEmpty(filePath) && !StringUtils.isEmpty(fileName)) {
            String fileName_ch = this.fileName.split("\\.dym")[0] + ".zh_CN.dymx";
            this.metadata = new XMLFileHelper(filePath + fileName);
            if (FileUtils.checkFileExists(this.filePath + fileName_ch)) {
                this.metadata_cn = new XMLFileHelper(this.filePath + fileName_ch);
            }
        }

    }

    public DymInfo getDymInfo() {
        if (this.metadata == null) {
            return null;
        } else {
            DymInfo dymInfo = new DymInfo();
            dymInfo.setDmzip(this.dmZipFile);
            dymInfo.setId(this.metadata.getElementStringValue("Id"));
            dymInfo.setNumber(this.metadata.getElementStringValue("DesignMetas.DesignFormMeta.Number"));
            dymInfo.setName(this.metadata_cn.getElementStringValue("DesignMetas.DesignFormMetaL.Name"));
            dymInfo.setModeltype(this.metadata.getElementStringValue("DesignMetas.DesignFormMeta.ModelType"));
            dymInfo.setIsvcode(this.metadata.getElementStringValue("DesignMetas.DesignFormMeta.Isv"));
            dymInfo.setTabels(String.join(",", this.getFormTabels()));
            String devType = this.metadata.getElementStringValue("DesignMetas.DesignFormMeta.DevType");
            dymInfo.setDevtype(devType);
            dymInfo.setAppId(this.metadata.getElementStringValue("BizappId"));
            String parentId = this.metadata.getElementStringValue("DesignMetas.DesignFormMeta.ParentId");
            if (!StringUtils.isEmpty(devType) && ("1".equals(devType) || "2".equals(devType))) {
                QFilter filter = new QFilter("id", "=", parentId);
                DynamicObject[] bos_formmetas = BusinessDataServiceHelper.load("bos_formmeta", "id,number,name", filter.toArray());
                if (bos_formmetas != null && bos_formmetas.length > 0) {
                    DynamicObject dym = bos_formmetas[0];
                    dymInfo.setParentnumber(dym.getString("number"));
                    dymInfo.setParentname(dym.getString("name"));
                }
            }

            return dymInfo;
        }
    }

    public List<String> getFormTabels() {
        List<String> tables = new ArrayList();
        if (this.metadata == null) {
            return tables;
        } else {
            String modelType = this.metadata.getElementStringValue("DesignMetas.DesignFormMeta.ModelType");
            if (StringUtils.isEmpty(modelType)) {
                return tables;
            } else {
                String billTableName;
                if ("BillFormModel".equalsIgnoreCase(modelType)) {
                    billTableName = this.metadata.getElementStringValue("DesignMetas.DesignEntityMeta.DataXml.EntityMetadata.Items.BillEntity.TableName");
                    if (!StringUtils.isEmpty(billTableName) && !tables.contains(billTableName)) {
                        tables.add(billTableName);
                    }
                } else {
                    if (!"BaseFormModel".equalsIgnoreCase(modelType)) {
                        return tables;
                    }

                    billTableName = this.metadata.getElementStringValue("DesignMetas.DesignEntityMeta.DataXml.EntityMetadata.Items.BaseEntity.TableName");
                    if (!StringUtils.isEmpty(billTableName) && !tables.contains(billTableName)) {
                        tables.add(billTableName);
                    }
                }

                List<Element> entryEntityList = this.metadata.getElements("DesignMetas.DesignEntityMeta.DataXml.EntityMetadata.Items.EntryEntity");
                if (entryEntityList != null) {
                    Iterator var4 = entryEntityList.iterator();

                    while(var4.hasNext()) {
                        Element entryEntity = (Element)var4.next();
                        String tableName = this.metadata.getElementStringValue(entryEntity, "TableName");
                        if (!StringUtils.isEmpty(tableName) && !tables.contains(tableName)) {
                            tables.add(tableName);
                        }
                    }
                }

                List<Element> subEntryEntityList = this.metadata.getElements("DesignMetas.DesignEntityMeta.DataXml.EntityMetadata.Items.SubEntryEntity");
                if (subEntryEntityList != null) {
                    Iterator var10 = subEntryEntityList.iterator();

                    while(var10.hasNext()) {
                        Element subEntryEntity = (Element)var10.next();
                        String tableName = this.metadata.getElementStringValue(subEntryEntity, "TableName");
                        if (!StringUtils.isEmpty(tableName) && !tables.contains(tableName)) {
                            tables.add(tableName);
                        }
                    }
                }

                return tables;
            }
        }
    }

    public String getDmZipFile() {
        return this.dmZipFile;
    }

    public void setDmZipFile(String dmZipFile) {
        this.dmZipFile = dmZipFile;
    }

    public String getFilePath() {
        return this.filePath;
    }

    public void setFilePath(String filePath) {
        this.filePath = filePath;
    }

    public String getFileName() {
        return this.fileName;
    }

    public void setFileName(String fileName) {
        this.fileName = fileName;
    }

    public XMLFileHelper getMetadata() {
        return this.metadata;
    }

    public void setMetadata(XMLFileHelper metadata) {
        this.metadata = metadata;
    }

    public XMLFileHelper getMetadata_cn() {
        return this.metadata_cn;
    }

    public void setMetadata_cn(XMLFileHelper metadata_cn) {
        this.metadata_cn = metadata_cn;
    }
}
