package q1k2.base.common.ceco.plugin.formplugin;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import kd.bos.cache.CacheFactory;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.metadata.dynamicobject.DynamicObjectType;
import kd.bos.entity.datamodel.events.PackageDataEvent;
import kd.bos.fileservice.FileServiceFactory;
import kd.bos.form.*;
import kd.bos.form.control.EntryGrid;
import kd.bos.form.control.Toolbar;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.form.events.MessageBoxClosedEvent;
import kd.bos.form.plugin.AbstractFormPlugin;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.bos.metadata.devportal.AppMetadata;
import kd.bos.orm.query.QFilter;
import kd.bos.portal.util.SerializationUtils;
import kd.bos.servicehelper.AttachmentDto;
import kd.bos.servicehelper.AttachmentServiceHelper;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.devportal.AppMetaServiceHelper;
import kd.bos.util.StringUtils;
import kd.bos.zip.ZipFile;
import kd.sdk.plugin.Plugin;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.CellCopyPolicy;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import q1k2.base.common.ceco.enums.ErrorTypeEnum;
import q1k2.base.common.ceco.enums.ScriptTypeEnum;
import q1k2.base.common.ceco.helper.XMLFileHelper;
import q1k2.base.common.ceco.model.*;
import q1k2.base.common.ceco.model.xmlfilemodel.*;
import q1k2.base.common.utils.FileUtils;

import java.io.*;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.jar.JarFile;

/**
 * @BelongsProject: q1k2-cosmic
 * @BelongsPackage: q1k2.base.common.ceco.plugin.formplugin
 * @Author: yanhongwei
 * @CreateTime: 2024-06-26  18:17
 * @Description: TODO
 * @Version: 1.0
 */
public class IsvPatchpackFormPlugin extends AbstractFormPlugin implements Plugin {
    private static Log log = LogFactory.getLog(IsvPatchpackFormPlugin.class);
    private static final String BASEISVINFO = "isvinfo";
    private static final String BTNGETPTINFO = "btngetptinfo";
    private static final String BTNEXPORTDEVREPORT = "exportdevreport";
    private static final String KDPKGSFILE = "kdpkgs.xml";
    private static final String ENTRY_CLOUD = "entry_cloud";
    private static final String ENTRY_APP = "entry_app";
    private static final String ENTRY_ERR = "entry_err";
    private static final String ENTRY_JAR = "entry_jar";
    private static final String ENTRY_META = "entry_meta";
    private static final String ENTRY_SQL = "entry_sql";
    private static final String ENTRY_KS = "entry_ks";
    private static final String ENTRY_CR = "entry_cr";
    private static final String ENTRY_APPMENU = "entity_appmenu";
    private static final String ENTRY_SCHEDULE = "entry_schedule";
    private static final String ENTRY_WB = "entry_wb";
    private static final String DM = "dm";
    private static final String JAR = "jar";
    private static final String RESOURCE = "resource";
    private static final String CACHE_JARFILEPREFIX = "Cache_JarFilePrefix";
    private static final String CACHE_ISVCODES = "Cache_IsvCodes";
    private static final String CACHE_APPINFOS = "Cache_AppInfos";
    private static final String CACHE_FORMS = "Cache_Forms";

    public IsvPatchpackFormPlugin() {
    }

    public void registerListener(EventObject e) {
        super.registerListener(e);
        Toolbar toolbar = (Toolbar)this.getControl("tbmain");
        toolbar.addItemClickListener(this);
    }

    public void initialize() {
        super.initialize();
        EntryGrid entry_app = (EntryGrid)this.getControl("entry_app");
        entry_app.addPackageDataListener(this::packageData_App);
        EntryGrid entry_jar = (EntryGrid)this.getControl("entry_jar");
        entry_jar.addPackageDataListener(this::packageData_Jar);
        EntryGrid entry_meta = (EntryGrid)this.getControl("entry_meta");
        entry_meta.addPackageDataListener(this::packageData_Meta);
        EntryGrid entry_sql = (EntryGrid)this.getControl("entry_sql");
        entry_sql.addPackageDataListener(this::packageData_Sql);
    }

    private void packageData_Sql(PackageDataEvent event) {
        boolean iscustomctrl = event.getRowData().getBoolean("iscustomctrl");
        boolean isvsignsql = event.getRowData().getBoolean("isvsignsql");
        if (iscustomctrl) {
            this.setEntryRowBackColor("entry_sql", event.getRowIndex(), "#FFFFCD");
        }

        if (isvsignsql) {
            this.setEntryRowBackColor("entry_sql", event.getRowIndex(), "#FFEFD5");
        }

    }

    private void packageData_Meta(PackageDataEvent event) {
        boolean isextend = event.getRowData().getBoolean("isextend");
        if (isextend) {
            this.setEntryRowBackColor("entry_meta", event.getRowIndex(), "#FFFFCD");
        }

    }

    private void packageData_App(PackageDataEvent event) {
        boolean isextendapp = event.getRowData().getBoolean("isextendapp");
        if (isextendapp) {
            this.setEntryRowBackColor("entry_app", event.getRowIndex(), "#FFFFCD");
        }

    }

    private void packageData_Jar(PackageDataEvent event) {
        boolean istrd = event.getRowData().getBoolean("istrd");
        if (istrd) {
            this.setEntryRowBackColor("entry_jar", event.getRowIndex(), "#FFFFCD");
        }

    }

    public void itemClick(ItemClickEvent evt) {
        super.itemClick(evt);
        switch (evt.getItemKey()) {
            case "btngetptinfo":
                this.getView().showConfirm("是否确定解析补丁包内容？", MessageBoxOptions.YesNo, new ConfirmCallBackListener("CallBack_getPatchpackInfo", this));
                break;
            case "exportdevreport":
                try {
                    this.click_exportdevreport();
                } catch (IOException var7) {
                    log.error("出错：" + var7.getMessage());
                }
                break;
            case "barimpapp":
                FormShowParameter showParameter = new FormShowParameter();
                showParameter.setFormId("bos_devp_installapp");
                showParameter.getOpenStyle().setShowType(ShowType.Modal);
                Map<String, Object> customParams = new HashMap();
                customParams.put("bizcloudid", "3UH70CJ5G4O/");
                showParameter.setCustomParams(customParams);
                this.getView().showForm(showParameter);
        }

    }

    public void closedCallBack(ClosedCallBackEvent closedCallBackEvent) {
        super.closedCallBack(closedCallBackEvent);
        switch (closedCallBackEvent.getActionId()) {
            case "callBack_uploadexcel":
                Map returnData = (Map)closedCallBackEvent.getReturnData();
                if (returnData != null && returnData.containsKey("tempfile")) {
                    String tempfile = (String)returnData.get("tempfile");

                    try {
                        this.exportDevReport(tempfile);
                    } catch (Exception var8) {
                        log.error("生成报告出错：" + var8.getMessage());
                        this.getView().showTipNotification("生成报告出错：" + var8.getMessage());
                    }
                }
            default:
        }
    }

    public void confirmCallBack(MessageBoxClosedEvent messageBoxClosedEvent) {
        super.confirmCallBack(messageBoxClosedEvent);
        switch (messageBoxClosedEvent.getCallBackId()) {
            case "CallBack_getPatchpackInfo":
                if (MessageBoxResult.Yes.equals(messageBoxClosedEvent.getResult())) {
                    try {
                        this.getPatchpackInfo();
                    } catch (Exception var6) {
                        this.getView().showTipNotification("解析补丁包出错：" + var6.getMessage());
                        log.error("解析补丁包出错：" + var6.getMessage());
                    }
                }
            default:
        }
    }

    private void exportDevReport(String excelFile) throws IOException, InvalidFormatException {
        if (!StringUtils.isEmpty(excelFile) && FileUtils.checkFileExists(excelFile)) {
            File file = new File(excelFile);
            if (file != null && file.exists() && file.isFile()) {
                XSSFWorkbook workbook = new XSSFWorkbook(file);

                try {
                    int sheetCount = workbook.getNumberOfSheets();

                    for(int i = 0; i < sheetCount; ++i) {
                        XSSFSheet sheet = workbook.getSheetAt(i);
                        this.exportSheet(sheet);
                    }

                    this.downloadDevReport(workbook);
                    FileUtils.deleteFile(excelFile, true);
                } finally {
                    workbook.close();
                }
            }
        }
    }

    private void downloadDevReport(XSSFWorkbook workbook) throws IOException {
        if (workbook != null) {
            DynamicObject appDym = (DynamicObject)this.getModel().getValue("isvinfo");
            String name = appDym.getString("name");
            String fileName = String.format("星空·旗舰版ISV产品开发验收报告-%s.xlsx", name);
            OutputStream os = new ByteArrayOutputStream();
            workbook.write(os);
            InputStream is = new ByteArrayInputStream(((ByteArrayOutputStream)os).toByteArray());

            try {
                String url = CacheFactory.getCommonCacheFactory().getTempFileCache().saveAsUrl(fileName, is, 10000);
                this.getView().download(url);
            } catch (Exception var11) {
            } finally {
                is.close();
                os.close();
            }

        }
    }

    private void click_exportdevreport() throws IOException {
        FormShowParameter showParameter = new FormShowParameter();
        showParameter.setFormId("isvtool_uploadexcel");
        showParameter.getOpenStyle().setShowType(ShowType.Modal);
        showParameter.setCloseCallBack(new CloseCallBack(this, "callBack_uploadexcel"));
        this.getView().showForm(showParameter);
    }

    private void exportSheet(XSSFSheet sheet) {
        if (sheet != null) {
            int firstRowIndex = sheet.getFirstRowNum();
            int lastRowIndex = sheet.getLastRowNum();

            int addRow;
            for(int rowIndex = firstRowIndex; rowIndex <= lastRowIndex; rowIndex = rowIndex + addRow + 1) {
                XSSFRow row = sheet.getRow(rowIndex);
                this.setExcelCellValue(sheet, row);
                int newLastRowIndex = sheet.getLastRowNum();
                addRow = newLastRowIndex - lastRowIndex;
                lastRowIndex = newLastRowIndex;
            }

        }
    }

    private void setExcelCellValue(XSSFSheet sheet, XSSFRow row) {
        if (row != null) {
            int rowIndex = row.getRowNum();
            ExcelRowInfo excelRowInfo = new ExcelRowInfo(row);
            String entryKey = excelRowInfo.getEntryKey();
            List<ExcelCellInfo> cellList = excelRowInfo.getCellList();
            if (StringUtils.isEmpty(entryKey)) {
                Iterator var7 = cellList.iterator();

                while(var7.hasNext()) {
                    ExcelCellInfo cell = (ExcelCellInfo)var7.next();
                    int columnIndex = cell.getColumnIndex();
                    String baseDataKey = cell.getBaseDataKey();
                    String fieldKey = cell.getFieldKey();
                    if (!StringUtils.isEmpty(fieldKey)) {
                        if (StringUtils.isEmpty(baseDataKey)) {
                            row.getCell(columnIndex).setCellValue((String)this.getModel().getValue(fieldKey));
                        } else {
                            DynamicObject dym = (DynamicObject)this.getModel().getValue(baseDataKey);
                            row.getCell(columnIndex).setCellValue(dym.getString(fieldKey));
                        }
                    }
                }
            } else {
                int entryRowCount = this.getModel().getEntryRowCount(entryKey);

                for(int i = 0; i < entryRowCount; ++i) {
                    XSSFRow newRow;
                    if (i != 0) {
                        int insertRowIndex = rowIndex + i;
                        sheet.shiftRows(insertRowIndex, sheet.getLastRowNum(), 1, true, true);
                        sheet.copyRows(rowIndex, rowIndex, insertRowIndex, new CellCopyPolicy());
                        newRow = sheet.getRow(insertRowIndex);
                    } else {
                        newRow = row;
                    }

                    Iterator var23 = cellList.iterator();

                    while(var23.hasNext()) {
                        ExcelCellInfo cell = (ExcelCellInfo)var23.next();
                        int columnIndex = cell.getColumnIndex();
                        String baseDataKey = cell.getBaseDataKey();
                        String fieldKey = cell.getFieldKey();
                        String cellOldValue = cell.getCellOldValue();
                        XSSFCell newCell = newRow.getCell(columnIndex);
                        if (newCell == null) {
                            newCell = newRow.createCell(columnIndex);
                        }

                        String newCellValue;
                        if (StringUtils.isEmpty(fieldKey)) {
                            newCellValue = newCell.getStringCellValue();
                            if (!StringUtils.isEmpty(cellOldValue) && !cellOldValue.equalsIgnoreCase(newCellValue)) {
                                newCell.setCellValue(cellOldValue);
                            }
                        } else {
                            newCellValue = "";
                            if (!StringUtils.isEmpty(cell.getEntryKey())) {
                                if (this.getModel().getProperty(fieldKey).getPropertyType().toString().equals("boolean")) {
                                    newCellValue = (Boolean)this.getModel().getValue(fieldKey, i) ? "是" : "否";
                                } else {
                                    newCellValue = this.getModel().getValue(fieldKey, i).toString();
                                }
                            } else if (StringUtils.isEmpty(baseDataKey)) {
                                newCellValue = this.getModel().getValue(fieldKey).toString();
                            } else {
                                DynamicObject dym = (DynamicObject)this.getModel().getValue(baseDataKey);
                                newCellValue = dym.getString(fieldKey);
                            }

                            newCell.setCellValue(newCellValue);
                        }
                    }
                }
            }

        }
    }

    private String downloadAttachmentFile(String tempPath, String attpanelKey) throws IOException {
        if (!StringUtils.isEmpty(attpanelKey) && !StringUtils.isEmpty(tempPath)) {
            List<Map<String, Object>> attachments = AttachmentServiceHelper.getAttachments(this.getView().getEntityId(), this.getModel().getValue("id"), attpanelKey);
            if (attachments != null && !attachments.isEmpty()) {
                Map<String, Object> tempAtta = (Map)attachments.get(0);
                String attname = (String)tempAtta.get("name");
                this.getModel().setValue("ptfilename", attname);
                Object attPkId = tempAtta.get("attPkId");
                AttachmentDto amd = AttachmentServiceHelper.getAttachmentInfoByAttPk(attPkId);
                String sourcePath = amd.getResourcePath();
                sourcePath = FileServiceFactory.getAttachmentFileService().getFileServiceExt().getRealPath(sourcePath);
                InputStream in = FileServiceFactory.getAttachmentFileService().getInputStream(sourcePath);
                String tmpPatchPackZipFile = tempPath + File.separator + attname;
                FileUtils.copyInputStreamToFile(in, tmpPatchPackZipFile);
                return tmpPatchPackZipFile;
            } else {
                return "";
            }
        } else {
            return "";
        }
    }

    private boolean checkBillStatus() {
        String attpanelKey = "attachmentpanel";
        List<Map<String, Object>> attachments = AttachmentServiceHelper.getAttachments(this.getView().getEntityId(), this.getModel().getValue("id"), attpanelKey);
        if (attachments != null && !attachments.isEmpty()) {
            String billstatus = (String)this.getModel().getValue("billstatus");
            if (!"A".equalsIgnoreCase(billstatus)) {
                this.getView().showTipNotification("单据已经提交或审核，不能修改");
                return false;
            } else {
                return true;
            }
        } else {
            this.getView().showTipNotification("请先上传附件，并保存单据");
            return false;
        }
    }

    private void patchpackDataInit() {
        this.getModel().deleteEntryData("entry_cloud");
        this.getModel().deleteEntryData("entry_app");
        this.getModel().deleteEntryData("entry_err");
        this.getModel().deleteEntryData("entry_jar");
        this.getModel().deleteEntryData("entry_meta");
        this.getModel().deleteEntryData("entry_sql");
        this.getModel().deleteEntryData("entry_ks");
        this.getModel().deleteEntryData("entry_cr");
        this.getModel().deleteEntryData("entity_appmenu");
        this.getModel().deleteEntryData("entry_schedule");
        this.getModel().deleteEntryData("entry_wb");
    }

    private void updateEntry() {
        this.getView().updateView("entry_cloud");
        this.getView().updateView("entry_app");
        this.getView().updateView("entry_err");
        this.getView().updateView("entry_jar");
        this.getView().updateView("entry_meta");
        this.getView().updateView("entry_sql");
        this.getView().updateView("entry_ks");
        this.getView().updateView("entry_cr");
        this.getView().updateView("entity_appmenu");
        this.getView().updateView("entry_schedule");
        this.getView().updateView("entry_wb");
    }

    private void parsingPTFile_Jar(String unZipDirPath, Element kdpkgElement) throws IOException {
        if (kdpkgElement != null && !StringUtils.isEmpty(unZipDirPath)) {
            if (!unZipDirPath.endsWith(File.separator)) {
                unZipDirPath = unZipDirPath + File.separator;
            }

            String resourceID = kdpkgElement.element("ID").getStringValue();
            String resourceName = kdpkgElement.element("name").getStringValue();
            String resourcePatch = kdpkgElement.element("sourcePath").getStringValue();
            if (!StringUtils.isEmpty(resourceName) && !StringUtils.isEmpty(resourcePatch)) {
                if (!resourcePatch.endsWith(File.separator)) {
                    resourcePatch = resourcePatch + File.separator;
                }

                String jarFileName = unZipDirPath + resourcePatch + resourceName;
                if (jarFileName.toLowerCase().endsWith(".zip")) {
                    String unJarFilePath = jarFileName.split("\\.zip")[0];
                    ZipFile zipFile = new ZipFile(jarFileName);
                    zipFile.extractAll(unJarFilePath);
                    this.readFile_jar(resourceName, unJarFilePath);
                }
            } else {
                this.addPatchErrMsg(new PatchErrorInfo(ErrorTypeEnum.ERRTYPE_ERROR, "配置错误", String.format("补丁包配置文件%s错误：资源配置错误，ID=%s", "kdpkgs.xml", resourceID)));
            }
        }
    }

    private void readFile_jar(String jarZipName, String unJarFilePath) throws IOException {
        if (!StringUtils.isEmpty(jarZipName) && !StringUtils.isEmpty(unJarFilePath)) {
            if (!unJarFilePath.endsWith(File.separator)) {
                unJarFilePath = unJarFilePath + File.separator;
            }

            File file = new File(unJarFilePath);
            if (file.exists()) {
                File[] jarFileList = file.listFiles();
                File[] var5 = jarFileList;
                int var6 = jarFileList.length;

                for(int var7 = 0; var7 < var6; ++var7) {
                    File jarFile = var5[var7];
                    if (jarFile.isDirectory()) {
                        this.readFile_jar(jarZipName, unJarFilePath + jarFile.getName());
                    } else if (jarFile.isFile() && jarFile.getName().toLowerCase().endsWith(".jar")) {
                        String jarName = jarFile.getName();
                        JarFile jarPack = new JarFile(jarFile);
                        JarInfo jarInfo = new JarInfo();
                        jarInfo.setJarzip(jarZipName);
                        jarInfo.setJarName(jarFile.getName());
                        jarInfo.setIsvsignjar(jarName.toLowerCase().startsWith("amk-isv"));
                        jarInfo.setIstrd(this.chkJarIsTrd(jarName));
                        jarInfo.setJarVersion(FileUtils.getJarVersion(jarPack));
                        jarInfo.setPackagePath(FileUtils.getJarPackagePath(jarPack));
                        this.newEntryData_jar(jarInfo);
                        jarPack.close();
                    }
                }

            }
        }
    }

    private boolean chkJarIsTrd(String jarName) {
        if (StringUtils.isEmpty(jarName)) {
            return false;
        } else {
            List<String> jarFilePrefix = this.getJarFilePrefix();
            Iterator var3 = jarFilePrefix.iterator();

            String prefix;
            do {
                if (!var3.hasNext()) {
                    return true;
                }

                prefix = (String)var3.next();
            } while(!jarName.toLowerCase().startsWith(prefix.toLowerCase()));

            return false;
        }
    }

    private void parsingPTFile_DM(String unZipDirPath, Element kdpkgElement) throws IOException, DocumentException {
        if (kdpkgElement != null && !StringUtils.isEmpty(unZipDirPath)) {
            if (!unZipDirPath.endsWith(File.separator)) {
                unZipDirPath = unZipDirPath + File.separator;
            }

            String resourceID = kdpkgElement.element("ID").getStringValue();
            String resourceName = kdpkgElement.element("name").getStringValue();
            String resourcePatch = kdpkgElement.element("sourcePath").getStringValue();
            if (!StringUtils.isEmpty(resourceName) && !StringUtils.isEmpty(resourcePatch)) {
                if (!resourcePatch.endsWith(File.separator)) {
                    resourcePatch = resourcePatch + File.separator;
                }

                String dmFileName = unZipDirPath + resourcePatch + resourceName;
                String undmFilePath = dmFileName.split("\\.zip")[0];
                ZipFile zipFile = new ZipFile(dmFileName);
                zipFile.extractAll(undmFilePath);
                String xml_DataModelFileName = undmFilePath + File.separator + "datamodel" + File.separator + "datamodel.xml";
                if (!FileUtils.checkFileExists(xml_DataModelFileName)) {
                    this.addPatchErrMsg(new PatchErrorInfo(ErrorTypeEnum.ERRTYPE_ERROR, "配置错误", String.format("%s 中配置文件datamodel.xml不存在", resourceName)));
                } else {
                    XMLFileHelper xml_DataModel = new XMLFileHelper(xml_DataModelFileName);
                    Element datamodel_root = xml_DataModel.getRootElement();
                    String datamodel_ver = datamodel_root.element("ver").getStringValue();
                    Element datamodel_app = datamodel_root.element("app");
                    List<Element> appItemList = xml_DataModel.getElements(datamodel_app, "item");
                    Iterator var15 = appItemList.iterator();

                    while(true) {
                        while(var15.hasNext()) {
                            Element item = (Element)var15.next();
                            String appId = item.attributeValue("id");
                            String appName = item.attributeValue("name");
                            String appPatch = item.attributeValue("path");
                            if (!StringUtils.isEmpty(appId) && !StringUtils.isEmpty(appName) && !StringUtils.isEmpty(appPatch)) {
                                appPatch = undmFilePath + File.separator + "datamodel" + File.separator + datamodel_ver + File.separator + appPatch;
                                if (!appPatch.endsWith(File.separator)) {
                                    appPatch = appPatch + File.separator;
                                }

                                String metaDataPath = appPatch + "metadata" + File.separator;
                                this.parsingPTFile_metaData(resourceName, metaDataPath);
                                String xml_AppFileName = appPatch + appId + ".xml";
                                if (!FileUtils.checkFileExists(xml_AppFileName)) {
                                    this.addPatchErrMsg(new PatchErrorInfo(ErrorTypeEnum.ERRTYPE_ERROR, "配置错误", String.format("%s 应用 %s 的配置文件 %s.xml 不存在", resourceName, appName, appId)));
                                } else {
                                    boolean isvsignsql = "amk_lic_group".equalsIgnoreCase(appId);
                                    this.parsingPTFile_sqlfile(xml_AppFileName, resourceName, appPatch, appId, isvsignsql);
                                }
                            } else {
                                this.addPatchErrMsg(new PatchErrorInfo(ErrorTypeEnum.ERRTYPE_ERROR, "配置错误", String.format("%s 中配置文件datamodel.xml的应用 %s 配置错误", resourceName, appName)));
                            }
                        }

                        return;
                    }
                }
            } else {
                this.addPatchErrMsg(new PatchErrorInfo(ErrorTypeEnum.ERRTYPE_ERROR, "配置错误", String.format("补丁包配置文件%s错误：资源配置错误，ID=%s", "kdpkgs.xml", resourceID)));
            }
        }
    }

    private void parsingPTFile_sqlfile(String xml_AppFileName, String dmzip, String appPatch, String appId, boolean isvsignsql) throws DocumentException, IOException {
        if (!StringUtils.isEmpty(xml_AppFileName) && !StringUtils.isEmpty(dmzip) && !StringUtils.isEmpty(appPatch)) {
            XMLFileHelper appxml = new XMLFileHelper(xml_AppFileName);
            Element rootElement = appxml.getRootElement();
            List<Element> sqlFileList = rootElement.elements("SqlScript");
            Iterator var9 = sqlFileList.iterator();

            while(var9.hasNext()) {
                Element sqlFile = (Element)var9.next();
                String type = sqlFile.attributeValue("Type");
                String dbkey = sqlFile.attributeValue("dbkey");
                String sqlFileName = sqlFile.getStringValue();
                DynamicObjectCollection entryColl = this.getModel().getEntryEntity("entry_sql");
                DynamicObjectType treeEntryType = entryColl.getDynamicObjectType();
                DynamicObject entryObj = new DynamicObject(treeEntryType);
                entryObj.set("dmzip_sql", dmzip);
                entryObj.set("sqltype", type);
                entryObj.set("execdbroute", dbkey);
                entryObj.set("sqlfilename", sqlFileName);
                entryObj.set("appid_sql", appId);
                entryObj.set("isvsignsql", isvsignsql);
                entryObj.set("iscustomctrl", FileUtils.isCustomCtrlSql(appPatch + sqlFileName));
                entryColl.add(entryObj);
                this.getModel().updateEntryCache(entryColl);
            }

        }
    }

    private void parsingPTFile_metaData(String dmzip, String metaDataPath) throws DocumentException, IOException {
        if (!StringUtils.isEmpty(metaDataPath)) {
            File file = new File(metaDataPath);
            if (file.exists()) {
                File[] metaDataList = file.listFiles();
                File[] var5 = metaDataList;
                int var6 = metaDataList.length;

                for(int var7 = 0; var7 < var6; ++var7) {
                    File metaDataFile = var5[var7];
                    if (metaDataFile.isFile()) {
                        String fileName = metaDataFile.getName();
                        String fileExtName = FileUtils.getFileExtensionName(fileName);
                        switch (fileExtName.toLowerCase()) {
                            case "cld":
                                this.readFile_cld(dmzip, metaDataPath, fileName);
                                break;
                            case "app":
                                this.readFile_app(dmzip, metaDataPath, fileName);
                                break;
                            case "dym":
                                this.readFile_dym(dmzip, metaDataPath, fileName);
                                break;
                            case "ks":
                                this.readFile_ks(dmzip, metaDataPath, fileName);
                                break;
                            case "cr":
                                this.readFile_cr(dmzip, metaDataPath, fileName);
                                break;
                            case "schdata":
                                this.readFile_Schdata(dmzip, metaDataPath, fileName);
                                break;
                            case "wb":
                                this.readFile_wb(dmzip, metaDataPath, fileName);
                        }
                    }
                }

            }
        }
    }

    private String getCache_IsvCodes() {
        return this.getView().getPageCache().get("Cache_IsvCodes");
    }

    private void setCache_IsvCodes(String isvCode) {
        if (!StringUtils.isEmpty(isvCode)) {
            String cache_isvCodes = this.getCache_IsvCodes();
            if (StringUtils.isEmpty(cache_isvCodes)) {
                cache_isvCodes = isvCode;
            } else if (!cache_isvCodes.contains(isvCode)) {
                cache_isvCodes = cache_isvCodes + "," + isvCode;
            }

            this.getView().getPageCache().put("Cache_IsvCodes", cache_isvCodes);
        }
    }

    private List<String> getJarFilePrefix() {
        String cacheText = this.getView().getPageCache().get("Cache_JarFilePrefix");
        return (List)(StringUtils.isEmpty(cacheText) ? new ArrayList() : Arrays.asList(cacheText.split(",")));
    }

    private void setJarFilePrefix(String isvCode, String cloudID) {
        if (!StringUtils.isEmpty(isvCode)) {
            String prefix = "";
            List<String> jarFilePrefix = this.getJarFilePrefix();
            if (isvCode.equalsIgnoreCase("kingdee")) {
                if (StringUtils.isEmpty(cloudID)) {
                    return;
                }

                prefix = cloudID.toLowerCase() + "-";
            } else {
                prefix = isvCode + "-";
            }

            List<String> newJarFilePrefix = new ArrayList(jarFilePrefix);
            if (!newJarFilePrefix.contains(prefix)) {
                newJarFilePrefix.add(prefix);
            }

            prefix = prefix.replace("-", "_");
            if (!newJarFilePrefix.contains(prefix)) {
                newJarFilePrefix.add(prefix);
            }

            String cacheText = String.join(",", newJarFilePrefix);
            this.getView().getPageCache().put("Cache_JarFilePrefix", cacheText);
        }
    }

    private void readFile_ks(String dmzip, String filePath, String fileName) throws DocumentException, IOException {
        if (!StringUtils.isEmpty(filePath) && !StringUtils.isEmpty(fileName)) {
            XMLFileHelper metadata = new XMLFileHelper(filePath + fileName);
            Element rootElement = metadata.getRootElement();
            KScriptInfo ksInfo = new KScriptInfo();
            ksInfo.setDmzip(dmzip);
            ksInfo.setId(rootElement.element("Scripts").element("KScript").element("id").getStringValue());
            ksInfo.setNumber(rootElement.element("Scripts").element("KScript").element("number").getStringValue());
            ksInfo.setName(rootElement.element("Scripts").element("KScript").element("scriptname").getStringValue());
            String scriptType = "SCRIPTTYPE_" + rootElement.element("Scripts").element("KScript").element("scripttype").getStringValue();
            ksInfo.setScriptType(ScriptTypeEnum.valueOf(scriptType));
            ksInfo.setAppId(rootElement.element("Scripts").element("KScript").element("bizappid").getStringValue());
            ksInfo.setIsvCode(rootElement.element("Scripts").element("KScript").element("isv").getStringValue());
            ksInfo.setClassName(rootElement.element("Scripts").element("KScript").element("classname").getStringValue());
            this.newEntryData_ks(ksInfo);
        }
    }

    private void readFile_Schdata(String dmzip, String filePath, String fileName) throws DocumentException, IOException {
        if (!StringUtils.isEmpty(filePath) && !StringUtils.isEmpty(fileName)) {
            ScheduleXmlFileInfo scheduleXmlInfo = new ScheduleXmlFileInfo(dmzip, filePath, fileName);
            ScheduleInfo scheduleInfo = scheduleXmlInfo.getScheduleInfo();
            if (scheduleInfo == null) {
                this.addPatchErrMsg(new PatchErrorInfo(ErrorTypeEnum.ERRTYPE_ERROR, "解析出错", String.format("解析调度计划元数据文件出错：%s", fileName)));
            } else {
                this.newEntryData_Schedule(scheduleInfo);
            }
        }
    }

    private void readFile_wb(String dmzip, String filePath, String fileName) throws DocumentException, IOException {
        if (!StringUtils.isEmpty(filePath) && !StringUtils.isEmpty(fileName)) {
            WriteBackRuleXmlFileInfo wbXmlInfo = new WriteBackRuleXmlFileInfo(dmzip, filePath, fileName);
            WriteBackRuleInfo writeBackRuleInfo = wbXmlInfo.getWriteBackRuleInfo();
            if (writeBackRuleInfo == null) {
                this.addPatchErrMsg(new PatchErrorInfo(ErrorTypeEnum.ERRTYPE_ERROR, "解析出错", String.format("解析反写规则元数据文件出错：%s", fileName)));
            } else {
                this.newEntryData_wb(writeBackRuleInfo);
            }
        }
    }

    private void readFile_cr(String dmzip, String filePath, String fileName) throws DocumentException, IOException {
        if (!StringUtils.isEmpty(filePath) && !StringUtils.isEmpty(fileName)) {
            String fileName_ch = fileName.split("\\.cr")[0] + ".zh_CN.crx";
            if (!FileUtils.checkFileExists(filePath + fileName_ch)) {
                this.addPatchErrMsg(new PatchErrorInfo(ErrorTypeEnum.ERRTYPE_ERROR, "缺少文件", String.format("%s缺少转换规则的中文元数据文件%s", dmzip, fileName_ch)));
            } else {
                XMLFileHelper metadata = new XMLFileHelper(filePath + fileName);
                XMLFileHelper metadata_ch = new XMLFileHelper(filePath + fileName_ch);
                Element rootElement = metadata.getRootElement();
                Element rootElement_cn = metadata_ch.getRootElement();
                ConvertRuleInfo crInfo = new ConvertRuleInfo();
                crInfo.setDmzip(dmzip);
                crInfo.setId(rootElement.element("Id").getStringValue());
                crInfo.setName(rootElement_cn.element("DesignMetas").element("DesignConvertRuleMetaL").element("Name").getStringValue());
                crInfo.setDevType(rootElement.element("DesignMetas").element("DesignConvertRuleMeta").element("DevType").getStringValue());
                crInfo.setIsvCode(rootElement.element("DesignMetas").element("DesignConvertRuleMeta").element("Isv").getStringValue());
                crInfo.setSourceEntityNumber(rootElement.element("DesignMetas").element("DesignConvertRuleMeta").element("SourceEntityNumber").getStringValue());
                crInfo.setTargetEntityNumber(rootElement.element("DesignMetas").element("DesignConvertRuleMeta").element("TargetEntityNumber").getStringValue());
                this.newEntryData_cr(crInfo);
            }
        }
    }

    private void readFile_dym(String dmzip, String filePath, String fileName) throws DocumentException, IOException {
        if (!StringUtils.isEmpty(filePath) && !StringUtils.isEmpty(fileName)) {
            DymXmlFileInfo dymXmlFileInfo = new DymXmlFileInfo(dmzip, filePath, fileName);
            String fileName_ch = fileName.split("\\.dym")[0] + ".zh_CN.dymx";
            if (dymXmlFileInfo.getMetadata_cn() == null) {
                this.addPatchErrMsg(new PatchErrorInfo(ErrorTypeEnum.ERRTYPE_ERROR, "缺少文件", String.format("缺少表单的中文元数据文件%s", fileName_ch)));
            } else {
                DymInfo dymInfo = dymXmlFileInfo.getDymInfo();
                this.setDymCache(dymInfo);
                this.newEntryData_dym(dymInfo);
            }
        }
    }

    private void setDymCache(DymInfo dymInfo) {
        if (dymInfo != null) {
            JSONObject appCache = this.getDymCache();
            String number = dymInfo.getNumber();
            if (!appCache.containsKey(number)) {
                JSONObject jsonObject = (JSONObject) JSON.toJSON(dymInfo);
                appCache.put(number, jsonObject);
                this.getView().getPageCache().put("Cache_Forms", appCache.toJSONString());
            }

        }
    }

    private JSONObject getDymCache() {
        JSONObject dymInfos = new JSONObject();
        String cacheText = this.getView().getPageCache().get("Cache_Forms");
        if (!StringUtils.isEmpty(cacheText)) {
            dymInfos = JSON.parseObject(cacheText);
        }

        return dymInfos;
    }

    private void setAppCache(AppInfo appInfo) {
        if (appInfo != null) {
            JSONObject appCache = this.getAppCache();
            String appID = appInfo.getAppid();
            if (!appCache.containsKey(appID)) {
                JSONObject jsonObject = (JSONObject)JSON.toJSON(appInfo);
                appCache.put(appID, jsonObject);
                this.getView().getPageCache().put("Cache_AppInfos", appCache.toJSONString());
            }

        }
    }

    private JSONObject getAppCache() {
        JSONObject appInfos = new JSONObject();
        String cacheText = this.getView().getPageCache().get("Cache_AppInfos");
        if (!StringUtils.isEmpty(cacheText)) {
            appInfos = JSON.parseObject(cacheText);
        }

        return appInfos;
    }

    private void readFile_app(String dmzip, String filePath, String fileName) throws DocumentException, IOException {
        if (!StringUtils.isEmpty(filePath) && !StringUtils.isEmpty(fileName)) {
            AppXmlFileInfo appXmlInfo = new AppXmlFileInfo(dmzip, filePath, fileName);
            if (appXmlInfo.getMetadata_cn() == null) {
                String fileName_ch = fileName.split("\\.app")[0] + ".zh_CN.appx";
                this.addPatchErrMsg(new PatchErrorInfo(ErrorTypeEnum.ERRTYPE_ERROR, "缺少文件", String.format("缺少应用的中文元数据文件%s", fileName_ch)));
            } else {
                AppInfo appInfo = appXmlInfo.getAppInfo();
                if (appInfo == null) {
                    this.addPatchErrMsg(new PatchErrorInfo(ErrorTypeEnum.ERRTYPE_ERROR, "解析出错", String.format("解析应用元数据文件出错：%s", fileName)));
                } else {
                    List<AppMenuInfo> menuInfos = appXmlInfo.getAppMenuInfos();
                    Iterator var7 = menuInfos.iterator();

                    while(var7.hasNext()) {
                        AppMenuInfo menu = (AppMenuInfo)var7.next();
                        long pkID = appXmlInfo.getMenuParentPKId(menu.getParentId());
                        if (pkID <= 0L && !StringUtils.isEmpty(menu.getParentId()) && appInfo.isIsextendapp()) {
                            pkID = appXmlInfo.getMenuParentPKId(appInfo.getAppid());
                        }

                        menu.setParentPKID(pkID);
                        this.newEntryData_AppMenu(menu);
                    }

                    this.setCache_IsvCodes(appInfo.getAppisv());
                    this.setJarFilePrefix(appInfo.getAppisv(), "");
                    this.setAppCache(appInfo);
                    this.newEntryData_App(appInfo);
                }
            }
        }
    }

    private void readFile_cld(String dmzip, String filePath, String fileName) throws DocumentException, IOException {
        if (!StringUtils.isEmpty(filePath) && !StringUtils.isEmpty(fileName)) {
            String fileName_ch = fileName.split("\\.cld")[0] + ".zh_CN.cldx";
            if (!FileUtils.checkFileExists(filePath + fileName_ch)) {
                this.addPatchErrMsg(new PatchErrorInfo(ErrorTypeEnum.ERRTYPE_ERROR, "缺少文件", String.format("缺少云的中文元数据文件%s", fileName_ch)));
            } else {
                XMLFileHelper metadatacld = new XMLFileHelper(filePath + fileName);
                XMLFileHelper metadatacld_ch = new XMLFileHelper(filePath + fileName_ch);
                Element rootElement = metadatacld.getRootElement();
                Element rootElement_cn = metadatacld_ch.getRootElement();
                CloudInfo cloudInfo = new CloudInfo();
                cloudInfo.setDmzip(dmzip);
                cloudInfo.setCloudid(rootElement.element("Id").getStringValue());
                cloudInfo.setCloudnumber(rootElement.element("DesignMetas").element("DesignCloudMeta").element("Number").getStringValue());
                cloudInfo.setCloudisv(rootElement.element("DesignMetas").element("DesignCloudMeta").element("Isv").getStringValue());
                cloudInfo.setCloudname(rootElement_cn.element("DesignMetas").element("DesignCloudMetaL").element("Name").getStringValue());
                cloudInfo.setClouddesc(rootElement_cn.element("DesignMetas").element("DesignCloudMetaL").element("Description").getStringValue());
                this.setJarFilePrefix(cloudInfo.getCloudisv(), cloudInfo.getCloudnumber());
                this.newEntryData_Cld(cloudInfo);
            }
        }
    }

    private void newEntryData_ks(KScriptInfo ksInfo) {
        if (ksInfo != null && !StringUtils.isEmpty(ksInfo.getId()) && !StringUtils.isEmpty(ksInfo.getNumber())) {
            DynamicObjectCollection entryColl = this.getModel().getEntryEntity("entry_ks");
            DynamicObjectType treeEntryType = entryColl.getDynamicObjectType();
            DynamicObject entryObj = new DynamicObject(treeEntryType);
            entryObj.set("dmzip_ks", ksInfo.getDmzip());
            entryObj.set("scriptid", ksInfo.getId());
            entryObj.set("scriptnumber", ksInfo.getNumber());
            entryObj.set("scriptname", ksInfo.getName());
            entryObj.set("scripttype", ksInfo.getScriptType().getName());
            entryObj.set("kdbizappid", ksInfo.getAppId());
            entryObj.set("kdbizappnumber", ksInfo.getNumber());
            entryObj.set("ksisvcode", ksInfo.getIsvCode());
            entryObj.set("classname", ksInfo.getClassName());
            entryColl.add(entryObj);
            this.getModel().updateEntryCache(entryColl);
        }
    }

    private void newEntryData_dym(DymInfo dymInfo) {
        if (dymInfo != null && !StringUtils.isEmpty(dymInfo.getId()) && !StringUtils.isEmpty(dymInfo.getNumber())) {
            DynamicObjectCollection entryColl = this.getModel().getEntryEntity("entry_meta");
            DynamicObjectType entryType = entryColl.getDynamicObjectType();
            DynamicObject entryObj = new DynamicObject(entryType);
            entryObj.set("dmzip_dym", dymInfo.getDmzip());
            entryObj.set("metadatanumber", dymInfo.getNumber());
            entryObj.set("metadataname", dymInfo.getName());
            entryObj.set("modeltype", dymInfo.getModeltype());
            entryObj.set("bizappid", dymInfo.getAppId());
            entryObj.set("appnum", dymInfo.getAppnum());
            entryObj.set("isextend", dymInfo.IsExtend());
            entryObj.set("metadataisv", dymInfo.getIsvcode());
            entryObj.set("parentnumber", dymInfo.getParentnumber());
            entryObj.set("parentname", dymInfo.getParentname());
            entryObj.set("tables", dymInfo.getTabels());
            entryColl.add(entryObj);
            this.getModel().updateEntryCache(entryColl);
        }
    }

    private void newEntryData_cr(ConvertRuleInfo crInfo) {
        if (crInfo != null && !StringUtils.isEmpty(crInfo.getId())) {
            DynamicObjectCollection entryColl = this.getModel().getEntryEntity("entry_cr");
            DynamicObjectType entryType = entryColl.getDynamicObjectType();
            DynamicObject entryObj = new DynamicObject(entryType);
            entryObj.set("dmzip_cr", crInfo.getDmzip());
            entryObj.set("crid", crInfo.getId());
            entryObj.set("crname", crInfo.getName());
            entryObj.set("crisextend", crInfo.isExtend());
            entryObj.set("crisvcode", crInfo.getIsvCode());
            entryObj.set("sourceentitynumber", crInfo.getSourceEntityNumber());
            entryObj.set("targetentitynumber", crInfo.getTargetEntityNumber());
            entryColl.add(entryObj);
            this.getModel().updateEntryCache(entryColl);
        }
    }

    private void newEntryData_Schedule(ScheduleInfo scheduleInfo) {
        if (scheduleInfo != null && !StringUtils.isEmpty(scheduleInfo.getNumber())) {
            DynamicObjectCollection entryColl = this.getModel().getEntryEntity("entry_schedule");
            DynamicObjectType entryType = entryColl.getDynamicObjectType();
            DynamicObject entryObj = new DynamicObject(entryType);
            entryObj.set("dmzip_schedule", scheduleInfo.getDmzip());
            entryObj.set("number_schedule", scheduleInfo.getNumber());
            entryObj.set("name_schedule", scheduleInfo.getName());
            entryObj.set("repeatmode", scheduleInfo.getRepeatmode());
            entryObj.set("starttime", scheduleInfo.getStartTime());
            entryObj.set("endtime", scheduleInfo.getEndTime());
            entryObj.set("cron", scheduleInfo.getCron());
            entryObj.set("enable_schedule", scheduleInfo.isEnable());
            entryColl.add(entryObj);
            this.getModel().updateEntryCache(entryColl);
        }
    }

    private void newEntryData_wb(WriteBackRuleInfo writeBackRuleInfo) {
        if (writeBackRuleInfo != null && !StringUtils.isEmpty(writeBackRuleInfo.getID())) {
            DynamicObjectCollection entryColl = this.getModel().getEntryEntity("entry_wb");
            DynamicObjectType entryType = entryColl.getDynamicObjectType();
            DynamicObject entryObj = new DynamicObject(entryType);
            entryObj.set("dmzip_wb", writeBackRuleInfo.getDmzip());
            entryObj.set("id_wb", writeBackRuleInfo.getID());
            entryObj.set("name_wb", writeBackRuleInfo.getName());
            entryObj.set("sourceentitynumber_wb", writeBackRuleInfo.getSourceEntityNumber());
            entryObj.set("targetentitynumber_wb", writeBackRuleInfo.getTargetEntityNumber());
            entryObj.set("sysstatus", writeBackRuleInfo.getSysStatus());
            entryObj.set("devtype", writeBackRuleInfo.getDevType());
            entryObj.set("isvcode_wb", writeBackRuleInfo.getIsv());
            entryColl.add(entryObj);
            this.getModel().updateEntryCache(entryColl);
        }
    }

    private void newEntryData_AppMenu(AppMenuInfo menuInfo) {
        if (menuInfo != null && !StringUtils.isEmpty(menuInfo.getId())) {
            DynamicObjectCollection treeEntryColl = this.getModel().getEntryEntity("entity_appmenu");
            DynamicObjectType treeEntryType = treeEntryColl.getDynamicObjectType();
            DynamicObject entryObj = new DynamicObject(treeEntryType);
            entryObj.set("id", menuInfo.getPKID());
            entryObj.set("pid", menuInfo.getParentPKID());
            entryObj.set("menuid", menuInfo.getId());
            entryObj.set("parentid", menuInfo.getParentId());
            entryObj.set("menuname", menuInfo.getName());
            entryObj.set("menucaption", menuInfo.getCaption());
            entryObj.set("formnumber", menuInfo.getFormNumber());
            entryObj.set("formname", menuInfo.getFormName());
            entryObj.set("opentype", menuInfo.getOpenType());
            entryObj.set("menuvisible", menuInfo.isVisible());
            treeEntryColl.add(entryObj);
            this.getModel().updateEntryCache(treeEntryColl);
        }
    }

    private void newEntryData_App(AppInfo appInfo) {
        if (appInfo != null && !StringUtils.isEmpty(appInfo.getAppid()) && !StringUtils.isEmpty(appInfo.getAppnumber())) {
            DynamicObjectCollection entryColl = this.getModel().getEntryEntity("entry_app");
            DynamicObjectType entryType = entryColl.getDynamicObjectType();
            DynamicObject entryObj = new DynamicObject(entryType);
            entryObj.set("dmzip_app", appInfo.getDmzip());
            entryObj.set("appid", appInfo.getAppid());
            entryObj.set("appnumber", appInfo.getAppnumber());
            entryObj.set("appname", appInfo.getAppname());
            entryObj.set("appname_en", appInfo.getAppname_en());
            entryObj.set("appisv", appInfo.getAppisv());
            entryObj.set("isalluser", appInfo.isIsall());
            entryObj.set("isextendapp", appInfo.isIsextendapp());
            entryObj.set("dbroute", appInfo.getDbroute());
            entryColl.add(entryObj);
            this.getModel().updateEntryCache(entryColl);
        }
    }

    private void setEntryRowBackColor(String entryKey, int rowIndex, String color) {
        if (!StringUtils.isEmpty(entryKey) && !StringUtils.isEmpty(color) && rowIndex >= 0) {
            IClientViewProxy proxy = (IClientViewProxy)this.getView().getService(IClientViewProxy.class);
            ClientActions.createRowStyleBuilder().setRows(new int[]{rowIndex}).setBackColor(color).buildStyle().build().invokeControlMethod(proxy, entryKey);
        }
    }

    private void newEntryData_jar(JarInfo jarInfo) {
        if (jarInfo != null && !StringUtils.isEmpty(jarInfo.getJarName())) {
            DynamicObjectCollection entryColl = this.getModel().getEntryEntity("entry_jar");
            DynamicObjectType entryType = entryColl.getDynamicObjectType();
            DynamicObject entryObj = new DynamicObject(entryType);
            entryObj.set("jarzip", jarInfo.getJarzip());
            entryObj.set("jarname", jarInfo.getJarName());
            entryObj.set("jarver", jarInfo.getJarVersion());
            entryObj.set("packagepath", jarInfo.getPackagePath());
            entryObj.set("isvsignjar", jarInfo.isIsvsignjar());
            entryObj.set("istrd", jarInfo.isIstrd());
            entryColl.add(entryObj);
            this.getModel().updateEntryCache(entryColl);
        }
    }

    private void newEntryData_Cld(CloudInfo cloud) {
        if (cloud != null && !StringUtils.isEmpty(cloud.getCloudid()) && !StringUtils.isEmpty(cloud.getCloudnumber())) {
            DynamicObjectCollection entryColl = this.getModel().getEntryEntity("entry_cloud");
            DynamicObjectType entryType = entryColl.getDynamicObjectType();
            DynamicObject entryObj = new DynamicObject(entryType);
            entryObj.set("dmzip", cloud.getDmzip());
            entryObj.set("cloudid", cloud.getCloudid());
            entryObj.set("cloudnumber", cloud.getCloudnumber());
            entryObj.set("cloudname", cloud.getCloudname());
            entryObj.set("cloudname_en", cloud.getCloudname_en());
            entryObj.set("clouddesc", cloud.getClouddesc());
            entryObj.set("cloudisv", cloud.getCloudisv());
            entryColl.add(entryObj);
            this.getModel().updateEntryCache(entryColl);
        }
    }

    private void addPatchErrMsg(PatchErrorInfo err) {
        if (err != null && !StringUtils.isEmpty(err.getErrType()) && !StringUtils.isEmpty(err.getErrcode())) {
            DynamicObjectCollection entryColl = this.getModel().getEntryEntity("entry_err");
            DynamicObjectType entryType = entryColl.getDynamicObjectType();
            DynamicObject entryObj = new DynamicObject(entryType);
            entryObj.set("kind", err.getErrType());
            entryObj.set("errcode", err.getErrcode());
            entryObj.set("errmessage", err.getErrmessage());
            entryColl.add(entryObj);
            this.getModel().updateEntryCache(entryColl);
        }
    }

    private void parsingPatchpackFiles(String unZipDirPath) throws DocumentException, IOException, ParseException {
        if (!StringUtils.isEmpty(unZipDirPath)) {
            String kdpkgsFileName = unZipDirPath + File.separator + "kdpkgs.xml";
            KdpkgsXmlFileInfo kdpkgsXmlFileInfo = new KdpkgsXmlFileInfo(kdpkgsFileName);
            this.setCache_IsvCodes(kdpkgsXmlFileInfo.getIsvCode());
            this.setJarFilePrefix(kdpkgsXmlFileInfo.getIsvCode(), "");
            String version = kdpkgsXmlFileInfo.getVersion();
            String releasetime = kdpkgsXmlFileInfo.getReleasetime();
            this.getModel().setValue("version", version);
            if (!StringUtils.isEmpty(releasetime)) {
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                this.getModel().setValue("releasetime", sdf.parse(releasetime));
            } else {
                this.addPatchErrMsg(new PatchErrorInfo(ErrorTypeEnum.ERRTYPE_NOTSTANDARDIZED, "kdpkgs.xml", "没有发放时间属性（releaseTime）或值不规范"));
            }

            Element productElement = kdpkgsXmlFileInfo.getElement("product");
            String productName = productElement.attributeValue("name");
            String productName_CN = productElement.attributeValue("nameCN");
            String productDisplayName = productElement.attributeValue("displayName");
            this.getModel().setValue("ptcode", productName);
            this.getModel().setValue("ptname", productName_CN);
            this.getModel().setValue("displayname", productDisplayName);
            List<Element> appElements = productElement.elements("app");
            List<Element> kdpkgElements = kdpkgsXmlFileInfo.getElements("kdpkg");
            if (appElements != null && !appElements.isEmpty()) {
                Iterator var12 = appElements.iterator();

                while(var12.hasNext()) {
                    Element appEl = (Element)var12.next();
                    String resourceID = appEl.element("resource").getStringValue();
                    if (StringUtils.isEmpty(resourceID)) {
                        this.addPatchErrMsg(new PatchErrorInfo(ErrorTypeEnum.ERRTYPE_ERROR, "配置错误", String.format("补丁包配置文件%s错误，resource为空", "kdpkgs.xml")));
                        return;
                    }

                    Element kdpkgElement = kdpkgsXmlFileInfo.getElementFromID(kdpkgElements, resourceID);
                    if (kdpkgElement == null) {
                        this.addPatchErrMsg(new PatchErrorInfo(ErrorTypeEnum.ERRTYPE_ERROR, "配置错误", String.format("补丁包配置文件%s错误：资源ID %s 未配置", "kdpkgs.xml", resourceID)));
                        return;
                    }

                    String resourceType = kdpkgElement.element("type").getStringValue();
                    switch (resourceType.toLowerCase()) {
                        case "dm":
                            this.parsingPTFile_DM(unZipDirPath, kdpkgElement);
                            break;
                        case "jar":
                            this.parsingPTFile_Jar(unZipDirPath, kdpkgElement);
                        case "resource":
                            break;
                        default:
                            this.addPatchErrMsg(new PatchErrorInfo(ErrorTypeEnum.ERRTYPE_ERROR, "配置错误", String.format("补丁包配置文件%s错误：资源类型 %s 未定义", "kdpkgs.xml", resourceType)));
                    }
                }

                this.getModel().setValue("isvcode", this.getCache_IsvCodes());
            } else {
                this.addPatchErrMsg(new PatchErrorInfo(ErrorTypeEnum.ERRTYPE_ERROR, "配置错误", String.format("补丁包配置文件%s错误，读取配置无补丁内容", "kdpkgs.xml")));
            }
        }
    }

    private void getPatchpackInfo() throws Exception {
        if (this.checkBillStatus()) {
            this.patchpackDataInit();
            this.getView().getPageCache().remove("Cache_JarFilePrefix");
            this.getView().getPageCache().remove("Cache_IsvCodes");
            this.getView().getPageCache().remove("Cache_AppInfos");
            this.getView().getPageCache().remove("Cache_Forms");
            String attpanelKey = "attachmentpanel";
            String tempPath = FileUtils.getTempPath();
            log.info("tempPath=" + tempPath);

            try {
                String zipFileName = this.downloadAttachmentFile(tempPath, attpanelKey);
                String unZipDirPath = zipFileName.split("\\.zip")[0];
                ZipFile zipFile = new ZipFile(zipFileName);
                zipFile.extractAll(unZipDirPath);
                this.parsingPatchpackFiles(unZipDirPath);
                this.updateMetaDataApp();
                this.updateKScriptApp();
                this.updateMetaDataFormName_cr();
                this.updateMetaDataFormName_wb();
                this.updateEntry();
                this.getView().showSuccessNotification("补丁包解析完成。");
            } finally {
                FileUtils.deleteDirectory(tempPath);
            }

        }
    }

    private void updateMetaDataApp() {
        int metaRow = this.getModel().getEntryRowCount("entry_meta");

        for(int i = 0; i < metaRow; ++i) {
            String bizappid = (String)this.getModel().getValue("bizappid", i);
            AppInfo appInfo = this.getApp(bizappid);
            this.getModel().setValue("appnum", appInfo.getAppnumber(), i);
            this.getModel().setValue("app_name", appInfo.getAppname(), i);
        }

    }

    private void updateKScriptApp() {
        int metaRow = this.getModel().getEntryRowCount("entry_ks");

        for(int i = 0; i < metaRow; ++i) {
            String bizappid = (String)this.getModel().getValue("kdbizappid", i);
            AppInfo appInfo = this.getApp(bizappid);
            this.getModel().setValue("kdbizappnumber", appInfo.getAppnumber(), i);
            this.getModel().setValue("kdbizappname", appInfo.getAppname(), i);
        }

    }

    private void updateMetaDataFormName_cr() {
        int metaRow = this.getModel().getEntryRowCount("entry_cr");

        for(int i = 0; i < metaRow; ++i) {
            String sourceentitynumber = (String)this.getModel().getValue("sourceentitynumber", i);
            DymInfo dymInfo = this.getDymInfo(sourceentitynumber);
            this.getModel().setValue("sourceentityname", dymInfo.getName(), i);
            String targetentitynumber = (String)this.getModel().getValue("targetentitynumber", i);
            DymInfo dymInfo1 = this.getDymInfo(targetentitynumber);
            this.getModel().setValue("targetentityname", dymInfo1.getName(), i);
        }

    }

    private void updateMetaDataFormName_wb() {
        int metaRow = this.getModel().getEntryRowCount("entry_wb");

        for(int i = 0; i < metaRow; ++i) {
            String sourceentitynumber = (String)this.getModel().getValue("sourceentitynumber_wb", i);
            DymInfo dymInfo = this.getDymInfo(sourceentitynumber);
            this.getModel().setValue("sourceentityname_wb", dymInfo.getName(), i);
            String targetentitynumber = (String)this.getModel().getValue("targetentitynumber_wb", i);
            DymInfo dymInfo1 = this.getDymInfo(targetentitynumber);
            this.getModel().setValue("targetentityname_wb", dymInfo1.getName(), i);
        }

    }

    private DymInfo getDymInfo(String FormNumber) {
        DymInfo dymInfo = new DymInfo();
        if (StringUtils.isEmpty(FormNumber)) {
            return dymInfo;
        } else {
            JSONObject appCache = this.getDymCache();
            if (appCache.containsKey(FormNumber)) {
                dymInfo = (DymInfo) SerializationUtils.fromJsonString(appCache.getJSONObject(FormNumber).toJSONString(), DymInfo.class);
                return dymInfo;
            } else {
                if (StringUtils.isEmpty(dymInfo.getNumber())) {
                    QFilter filter = new QFilter("number", "=", FormNumber);
                    DynamicObject[] bos_formmetas = BusinessDataServiceHelper.load("bos_formmeta", "id,number,name", filter.toArray());
                    if (bos_formmetas != null && bos_formmetas.length > 0) {
                        DynamicObject dym = bos_formmetas[0];
                        dymInfo.setParentnumber(dym.getString("number"));
                        dymInfo.setParentname(dym.getString("name"));
                    }
                }

                return dymInfo;
            }
        }
    }

    private AppInfo getApp(String appid) {
        AppInfo appInfo = new AppInfo();
        if (StringUtils.isEmpty(appid)) {
            return appInfo;
        } else {
            JSONObject appCache = this.getAppCache();
            if (appCache.containsKey(appid)) {
                appInfo = (AppInfo)SerializationUtils.fromJsonString(appCache.getJSONObject(appid).toJSONString(), AppInfo.class);
                return appInfo;
            } else {
                if (StringUtils.isEmpty(appInfo.getAppid())) {
                    AppMetadata appMetadata = AppMetaServiceHelper.loadAppMetadataFromCacheById(appid, false);
                    if (appMetadata != null) {
                        appInfo.setAppid(appid);
                        appInfo.setAppnumber(appMetadata.getNumber());
                        appInfo.setAppname(appMetadata.getName().getLocaleValue_zh_CN());
                    }
                }

                return appInfo;
            }
        }
    }
}
