package q1k2.base.common.utils;

import kd.bos.util.StringUtils;

import java.io.*;
import java.nio.file.*;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.jar.Manifest;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @BelongsProject: q1k2-cosmic
 * @BelongsPackage: q1k2.base.common.utils
 * @Author: yanhongwei
 * @CreateTime: 2024-06-26  17:42
 * @Description: TODO
 * @Version: 1.0
 */
public class FileUtils {
    public FileUtils() {
    }

    public static boolean deleteFile(String fileName, boolean deleteOnExit) {
        if (StringUtils.isEmpty(fileName)) {
            return false;
        } else {
            File file = new File(fileName);
            if (file != null && file.exists() && file.isFile()) {
                if (deleteOnExit) {
                    file.deleteOnExit();
                } else {
                    file.delete();
                }

                return true;
            } else {
                return false;
            }
        }
    }

    public static boolean checkFileExists(String fileName) {
        if (StringUtils.isEmpty(fileName)) {
            return false;
        } else {
            File file = new File(fileName);
            return file.exists() && !file.isDirectory();
        }
    }

    public static String getTempPath() {
        String tmpDir = System.getProperty("java.io.tmpdir");
        String timeMillis = System.currentTimeMillis() + "";
        String tempPath = String.format("%s%s", tmpDir, timeMillis);
        File path = new File(tempPath);
        if (!path.exists()) {
            path.mkdirs();
        }

        if (!tempPath.endsWith(File.separator)) {
            tempPath = tempPath + File.separator;
        }

        return tempPath;
    }

    public static void deleteDirectory(String path) throws IOException {
        if (!StringUtils.isEmpty(path)) {
            Path directoryToDelete = Paths.get(path);
            deleteDirectory(directoryToDelete);
        }
    }

    public static void deleteDirectory(Path path) throws IOException {
        if (Files.notExists(path, new LinkOption[0])) {
            System.out.println("目录不存在: " + path);
        } else {
            if (Files.isDirectory(path, new LinkOption[0])) {
                DirectoryStream<Path> stream = Files.newDirectoryStream(path);
                Throwable var2 = null;

                try {
                    Iterator var3 = stream.iterator();

                    while(var3.hasNext()) {
                        Path file = (Path)var3.next();
                        if (Files.isDirectory(file, new LinkOption[]{LinkOption.NOFOLLOW_LINKS})) {
                            deleteDirectory(file);
                        } else {
                            Files.delete(file);
                        }
                    }
                } catch (Throwable var12) {
                    var2 = var12;
                    throw var12;
                } finally {
                    if (stream != null) {
                        if (var2 != null) {
                            try {
                                stream.close();
                            } catch (Throwable var11) {
                                var2.addSuppressed(var11);
                            }
                        } else {
                            stream.close();
                        }
                    }

                }
            } else {
                Files.delete(path);
            }

            Files.deleteIfExists(path);
        }
    }

    public static void copyInputStreamToFile(InputStream inputStream, File file) throws IOException {
        if (inputStream != null) {
            if (file.exists()) {
                file.delete();
            }

            FileOutputStream outputStream = new FileOutputStream(file);
            Throwable var3 = null;

            try {
                byte[] bytes = new byte[1024];

                int read;
                while((read = inputStream.read(bytes)) != -1) {
                    outputStream.write(bytes, 0, read);
                }
            } catch (Throwable var13) {
                var3 = var13;
                throw var13;
            } finally {
                if (outputStream != null) {
                    if (var3 != null) {
                        try {
                            outputStream.close();
                        } catch (Throwable var12) {
                            var3.addSuppressed(var12);
                        }
                    } else {
                        outputStream.close();
                    }
                }

            }

        }
    }

    public static void copyInputStreamToFile(InputStream inputStream, String fileName) throws IOException {
        if (inputStream != null && !StringUtils.isEmpty(fileName)) {
            File file = new File(fileName);
            copyInputStreamToFile(inputStream, file);
        }
    }

    public static String getFileExtensionName(File file) {
        return file != null && file.isFile() ? getFileExtensionName(file.getName()) : "";
    }

    public static String getFileExtensionName(String fileName) {
        if (StringUtils.isEmpty(fileName)) {
            return "";
        } else {
            int dotIndex = fileName.lastIndexOf(46);
            return dotIndex != -1 ? fileName.substring(dotIndex + 1) : "";
        }
    }

    public static String getJarVersion(JarFile jarFile) throws IOException {
        if (jarFile == null) {
            return "";
        } else {
            Manifest manifest = jarFile.getManifest();
            if (manifest == null) {
                return parseVersion(jarFile.getName());
            } else {
                String version = manifest.getMainAttributes().getValue("Implementation-Version");
                if (version == null || version.isEmpty()) {
                    version = parseVersion(jarFile.getName());
                }

                return version;
            }
        }
    }

    public static String parseVersion(String jarFilePath) {
        String[] pathParts = jarFilePath.split("\\\\");
        String jarFileName = pathParts[pathParts.length - 1];
        String regex = "\\d+\\.\\d+(\\.\\d+)?";
        Pattern pattern = Pattern.compile(regex);
        Matcher matcher = pattern.matcher(jarFileName);
        return matcher.find() ? matcher.group() : "";
    }

    public static String getJarPackagePath(JarFile jarFile) {
        if (jarFile == null) {
            return "";
        } else {
            List<String> packagePaths = new ArrayList();
            Enumeration entries = jarFile.entries();

            while(entries.hasMoreElements()) {
                JarEntry entry = (JarEntry)entries.nextElement();
                String entryName = entry.getName();
                if (entryName.endsWith(".class")) {
                    String className = entryName.replace("/", ".").replace(".class", "");
                    packagePaths.add(className);
                }
            }

            return ComPreExtractorUtil.getCommonPrefixs(packagePaths);
        }
    }

    public static boolean isCustomCtrlSql(String sqlFileName) {
        boolean flag = Boolean.FALSE;

        try {
            BufferedReader br = new BufferedReader(new FileReader(sqlFileName));

            String line;
            while((line = br.readLine()) != null) {
                if (line.toUpperCase().contains("T_META_CTLSCHEMA")) {
                    flag = Boolean.TRUE;
                    break;
                }
            }

            br.close();
        } catch (IOException var4) {
            flag = Boolean.FALSE;
        }

        return flag;
    }
}
