package q1k2.base.common.utils;

import oadd.org.apache.calcite.avatica.org.apache.http.util.TextUtils;

import java.io.*;
import java.util.zip.*;

/**
 * @BelongsProject: q1k2-cosmic
 * @BelongsPackage: q1k2.base.common.utils
 * @Author: yanhongwei
 * @CreateTime: 2024-06-26  17:43
 * @Description: TODO
 * @Version: 1.0
 */
public class ZipUtil {
    public ZipUtil() {
    }

    public static void decompress(String zip, String destPath) throws Exception {
        if (!TextUtils.isEmpty(zip) && !TextUtils.isEmpty(destPath)) {
            File zipFile = new File(zip);
            if (!zipFile.exists()) {
                throw new FileNotFoundException("zip file is not exists");
            } else {
                File destFolder = new File(destPath);
                if (!destFolder.exists() && !destFolder.mkdirs()) {
                    throw new FileNotFoundException("destPath mkdirs is failed");
                } else {
                    ZipInputStream zis = null;
                    BufferedOutputStream bos = null;

                    try {
                        zis = new ZipInputStream(new BufferedInputStream(new FileInputStream(zip)));

                        while(true) {
                            ZipEntry ze;
                            while((ze = zis.getNextEntry()) != null) {
                                String filePath = destPath + File.separator + ze.getName();
                                if (ze.isDirectory()) {
                                    (new File(filePath)).mkdirs();
                                } else {
                                    ByteArrayOutputStream baos = new ByteArrayOutputStream();
                                    byte[] buffer = new byte[1024];

                                    int count;
                                    while((count = zis.read(buffer)) != -1) {
                                        baos.write(buffer, 0, count);
                                    }

                                    byte[] bytes = baos.toByteArray();
                                    File entryFile = new File(filePath);
                                    if (!entryFile.getParentFile().exists() && !entryFile.getParentFile().mkdirs()) {
                                        throw new FileNotFoundException("zip entry mkdirs is failed");
                                    }

                                    bos = new BufferedOutputStream(new FileOutputStream(entryFile));
                                    bos.write(bytes);
                                    bos.flush();
                                }
                            }

                            return;
                        }
                    } finally {
                        closeQuietly(zis);
                        closeQuietly(bos);
                    }
                }
            }
        } else {
            throw new IllegalArgumentException("zip or destPath is illegal");
        }
    }

    public static void compress(String srcPath, String destPath) throws Exception {
        if (!TextUtils.isEmpty(srcPath) && !TextUtils.isEmpty(destPath)) {
            File srcFile = new File(srcPath);
            if (!srcFile.exists()) {
                throw new FileNotFoundException("srcPath file is not exists");
            } else {
                File destFile = new File(destPath);
                if (destFile.exists() && !destFile.delete()) {
                    throw new IllegalArgumentException("destFile is exist and do not delete.");
                } else {
                    CheckedOutputStream cos = null;
                    ZipOutputStream zos = null;

                    try {
                        cos = new CheckedOutputStream(new FileOutputStream(destPath), new CRC32());
                        zos = new ZipOutputStream(cos);
                        zos.setLevel(9);
                        if (srcFile.isFile()) {
                            compressFile("", srcFile, zos);
                        } else if (srcFile.isDirectory()) {
                            compressFolder("", srcFile, zos);
                        }
                    } finally {
                        closeQuietly(zos);
                    }

                }
            }
        } else {
            throw new IllegalArgumentException("srcPath or destPath is illegal");
        }
    }

    private static void compressFolder(String prefix, File srcFolder, ZipOutputStream zos) throws IOException {
        String new_prefix = prefix + srcFolder.getName() + "/";
        File[] files = srcFolder.listFiles();
        if (files.length == 0) {
            compressFile(prefix, srcFolder, zos);
        } else {
            File[] var5 = files;
            int var6 = files.length;

            for(int var7 = 0; var7 < var6; ++var7) {
                File file = var5[var7];
                if (file.isFile()) {
                    compressFile(new_prefix, file, zos);
                } else if (file.isDirectory()) {
                    compressFolder(new_prefix, file, zos);
                }
            }
        }

    }

    private static void compressFile(String prefix, File src, ZipOutputStream zos) throws IOException {
        String relativePath = prefix + src.getName();
        if (src.isDirectory()) {
            relativePath = relativePath + "/";
        }

        ZipEntry entry = new ZipEntry(relativePath);
        zos.putNextEntry(entry);
        InputStream is = null;

        try {
            if (src.isFile()) {
                is = new FileInputStream(src);
                byte[] buffer = new byte[8192];
                int len = 0;

                while((len = is.read(buffer)) != -1) {
                    zos.write(buffer, 0, len);
                }
            }

            zos.closeEntry();
        } finally {
            closeQuietly(is);
        }

    }

    private static void closeQuietly(Closeable closeable) {
        try {
            if (closeable != null) {
                closeable.close();
            }
        } catch (IOException var2) {
        }

    }
}
